import React, { useEffect, useRef, useState } from 'react'
import classNames from 'classnames'
import createDimension, { useDimension } from 'dymension'
import Toast, { ToastProps } from 'react-bootstrap/Toast'
import './index.scss'

interface ToasterProps extends ToastProps {
    key?: string;
    header: React.ReactNode;
    body: React.ReactNode;
    className?: string;
    classEnter?: string;
    classExit?: string;
    bg?: 'primary' | 'success' | 'danger' | 'warning' | 'info' | 'light' | 'dark';
    placement?: 'top' | 'top-left' | 'top-right' | 'bottom' | 'bottom-left' | 'bottom-right';
    duration?: number;
    delay?: number;
    dismissible?: boolean;
    onMount?: () => void;
    onUnmount?: () => void;
    onDismissed?: () => void;
}

export const Toaster: React.FunctionComponent<ToasterProps> = ({ 
	header,
    body,
    bg,
    className,
    classEnter,
    classExit,
    duration,
    dismissible,
    onMount,
    onUnmount,
    onDismissed,
	...props
}) => {
    const {
        show,
        resolve,
    } = useDimension()

	const [pause, setPause] = useState(false)
	const observables = useRef<{
		start: number;
		duration: number;
	}>({
		start: Date.now(),
		duration: duration!,
	}).current

	useEffect(() => {
		observables.start = Date.now()
		observables.duration = duration!
	}, [duration])

    //@ts-ignore
    useEffect(() => {
		if(pause) {

			observables.duration = Math.max(0, observables.duration - (Date.now() - observables.start))

		} else if(observables.duration! > 0 && show) {

            const cleanup = setTimeout(() => {

				observables.duration = 0

                resolve(true)

            }, observables.duration)

            return () => clearTimeout(cleanup)
        }
        // eslint-disable-next-line react-hooks/exhaustive-deps
    }, [duration, show, pause])

    useEffect(() => {

        onMount && onMount()

        return () => onUnmount && onUnmount()

        // eslint-disable-next-line react-hooks/exhaustive-deps
    }, [onMount, onUnmount])

	const handleClose = () => {

		observables.duration = 0

		resolve(true)

		onDismissed && onDismissed()
	}

	const handleMouseEnter = () => {
		if(observables.duration! > 0) 
			setPause(true)
	}

	const handleMouseLeave = () => {
		if(observables.duration! > 0) {
			setPause(false)
            observables.start = Date.now()
        }
	}

    return (
        <Toast 
			className={classNames(className, { [classEnter!]: show, [classExit!]: !show })} 
			onClose={handleClose} 
			onMouseEnter={handleMouseEnter}
			onMouseLeave={handleMouseLeave}
            bg={bg}
			{...props}>
			<Toast.Header closeButton={dismissible}>
				{header}
                {duration! > 0 && (
                    <div className={classNames("toast-progress", {[`text-${bg}`]: bg !== 'light'})}>
                        <div className={classNames("toast-progress-bar", { pause })} style={{animationDuration: `${duration}ms`}}/>
                    </div>
                )}
			</Toast.Header>
			<Toast.Body className={classNames("", { "text-white": bg === "dark" })}>
				{body}
			</Toast.Body>
        </Toast>
    )
}

Toaster.defaultProps = {
    dismissible: false,
    duration: 6000,
    classEnter: 'rb-fade-toast-in',
    classExit: 'rb-fade-toast-out',
    bg: 'light'
}

const createNode = (parent?: any) => {
    if(typeof document !== "undefined") {
        return (parent || document.body).appendChild(document.createElement('div'))
    }
}

const toastContainer = createNode()

const setNodePosition = (position: string) => {

    const node = createNode(toastContainer)

    if(node) {
        node.id = `rb-toast-${position}`
    }

    return node
}

const container = ['top', 'bottom'].reduce<Record<string, any>>((result, primePosition) => {

    if(['top', 'bottom'].includes(primePosition)) {

        result[primePosition] = setNodePosition(primePosition)

    }

    return ['left', 'right'].reduce((result, position) => {

        const nodeProsition = `${primePosition}-${position}`

        result[nodeProsition] = setNodePosition(nodeProsition)

        return result

    }, result)

}, {})

export const isToastKeyExist = (key: string, placement?: ToasterProps["placement"]) => {

	if(typeof document !== "undefined") {
		const existNode = document.getElementById(key)
		return !!existNode === true && (placement ? document.getElementById(placement)?.contains(existNode) === true : true)
	}

	return false
}

export default createDimension(Toaster, {
    delay: ({ delay = 600 }) => delay,
    containerNode: ({ placement = 'top-right'}) => container[placement],
    manipulateWrapperNode: (wrapperNode, props) => {

        if(props.key && typeof document !== "undefined") {
            const existNode = document.getElementById(props.key)

            if(existNode) {

                wrapperNode = existNode

            } else {

                wrapperNode.id = props.key

            }
        }

        wrapperNode.className = 'rb-toast-wrapper'

        return wrapperNode
    }
})